/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Manuel Leuenberger
   co-programmer: ...
*/

#include "network_turret.h"
#include "model.h"
#include "world_entities/weapons/turret.h"

#include "state.h"
#include "playable.h"
#include "player.h"


#include "util/loading/factory.h"
#include "network_game_manager.h"
#include "util/loading/load_param.h"

#include "effects/explosion.h"

#include "weapons/targeting_turret.h"

CREATE_FACTORY(NetworkTurret, CL_NETWORK_TURRET);




/**
 * constructs and loads a NetworkTurret from a XML-element
 * @param root the XML-element to load from
 */
NetworkTurret::NetworkTurret(const TiXmlElement* root)
    : NPC(root)
{
  this->init();
  if (root != NULL)
    this->loadParams(root);
}


/**
 * standard deconstructor
 */
NetworkTurret::~NetworkTurret ()
{}


/**
 * initializes the NetworkTurret
 * @todo change this to what you wish
 */
void NetworkTurret::init()
{
  this->setClassID(CL_NETWORK_TURRET, "NetworkTurret");
  this->loadModel("models/network_turret_#.obj", 5);
  this->weapon = new TargetingTurret();
  this->weapon->loadModel("models/guns/turret2.obj", 10);

  this->setHealthMax(300);
  this->setHealth(300);

  this->weaponHolder.addNodeFlags(PNODE_PROHIBIT_DELETE_WITH_PARENT);

  this->weaponHolder.setRelCoor(0,25,0);
  this->weaponHolder.setParent(this);
}


/**
 * @brief loads a NetworkTurret from a XML-element
 * @param root the XML-element to load from
 * @todo make the class Loadable
 */
void NetworkTurret::loadParams(const TiXmlElement* root)
{
  // all the clases this Entity is directly derived from must be called in this way, to load all settings.
  NPC::loadParams(root);
}

/**
 * advances the NetworkTurret about time seconds
 * @param time the Time to step
 */
void NetworkTurret::tick(float dt)
{
  if(this->getHealth() > 0.0f
    ) // HACK <--- YOU ARE THE MOTHERFUCKER
  {
    if (likely(this->weapon != NULL))
    {
      //    this->left->tickW(dt);
      this->weapon->requestAction(WA_SHOOT);
    }
  }
}

/**
 * draws this worldEntity
 */
void NetworkTurret::draw () const
{
  WorldEntity::draw();

  if (this->weapon != NULL)
    this->weapon->draw();
}



/**
 *
 *
 */
void NetworkTurret::postSpawn ()
{}

/**
 *
 *
 */
void NetworkTurret::leftWorld ()
{}

void NetworkTurret::destroy(WorldEntity* killer)
{
  this->setAbsDirSoft(Quaternion(-90, Vector(0,0,1)), 90);
  Explosion::explode(this, Vector(10,10,10));

  this->toList(OM_DEAD);
}
